/*
 * This file is part of libphidget21
 *
 * Copyright 2006-2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see 
 * <http://www.gnu.org/licenses/>
 */

#define POWER_CALLBACKS

#include "stdafx.h"
#include "cphidgetmanager.h"
#include "cphidgetconstants.h"
#include "cphidgetlist.h"
#include "macusb.h"

// globals
static CFRunLoopRef				runLoopRef = NULL;
static IONotificationPortRef	gNotifyPort;
static io_iterator_t			gRawAddedIter;
static io_iterator_t			gRawRemovedIter;

#ifdef POWER_CALLBACKS
extern int pause_usb_traffic;
extern int usb_read_paused;
extern int usb_write_paused;

static IONotificationPortRef	notifyPortRef;
static io_connect_t				root_port; // a reference to the Root Power Domain IOService
static io_object_t				notifierObject; // notifier object, used to deregister later
#endif

int(CCONV *fptrSleep)(void *) = 0;
void *fptrSleepptr = NULL;
int(CCONV *fptrWakeup)(void *) = 0;
void *fptrWakeupptr = NULL;

int CCONV
CPhidget_set_OnWillSleep_Handler(int(CCONV *fptr)(void *userPtr), void *userPtr)
{
	fptrSleep = fptr;
	fptrSleepptr = userPtr;
	return EPHIDGET_OK;
}

int CCONV
CPhidget_set_OnWakeup_Handler(int(CCONV *fptr)(void *userPtr), void *userPtr)
{
	fptrWakeup = fptr;
	fptrWakeupptr = userPtr;
	return EPHIDGET_OK;
}

void freeString(char * cstr) {
    if (cstr != NULL) {
        free(cstr);
        cstr = NULL;
    }
}

UInt16	Swap16(void *p) {
    * (UInt16 *) p = CFSwapInt16LittleToHost(*(UInt16 *)p);
    return * (UInt16 *) p;
}

UInt32	Swap32(void *p) {
    * (UInt32 *) p = CFSwapInt32LittleToHost(*(UInt32 *)p);
    return * (UInt32 *) p;
}

UInt64	Swap64(void *p) {
    * (UInt64 *) p = CFSwapInt64LittleToHost(*(UInt64 *)p);
    return * (UInt64 *) p;
}

//for getting the device descriptor
int GetDescriptor(IOUSBDeviceInterface245 **deviceIntf, UInt8 descType, UInt8 descIndex, void *buf, UInt16 len) {
    IOUSBDevRequest req;
    IOReturn err;
    
    req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
    req.bRequest = kUSBRqGetDescriptor;
    req.wValue = (descType << 8) | descIndex;
    req.wIndex = 0;
    req.wLength = len;
    req.pData = buf;
    
    err = (*deviceIntf)->DeviceRequest(deviceIntf, &req);
    if (err != kIOReturnSuccess) {
		LOG(PHIDGET_LOG_WARNING, "GetDescriptor failed with error: 0x%08x",err);
        return -1;
    }
    return req.wLenDone;
}

//for getting the interface descriptor (HID Descriptor in this case)
int GetDescriptorFromInterface(IOUSBDeviceInterface245 **deviceIntf, UInt8 descType, UInt8 descIndex, UInt16 wIndex, void *buf, UInt16 len)
{
    IOUSBDevRequest req;
    IOReturn err;
    
    req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBInterface);
    req.bRequest = kUSBRqGetDescriptor;
    req.wValue = (descType << 8) | descIndex;
    req.wIndex = wIndex;
    req.wLength = len;
    req.pData = buf;
    
    err = (*deviceIntf)->DeviceRequest(deviceIntf, &req);
    if (err != kIOReturnSuccess) {
		LOG(PHIDGET_LOG_WARNING, "GetDescriptorFromInterface failed with error: 0x%08x",err);
        return -1;
    }
	
    return req.wLenDone;
}

//for getting String descriptors, such as serial number, manufacturer, etc.
int GetStringDescriptorMaster(void *deviceIntf, UInt8 descIndex, void *buf, UInt16 len, UInt16 lang, int isIOUSBDeviceInterface) {
    IOUSBDevRequest req;
    UInt8 		desc[256]; // Max possible descriptor length
    int stringLen;
    IOReturn err;
    if (lang == 0) // set default langID
        lang=0x0409;
    
	//this gets the descriptor length
    req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
    req.bRequest = kUSBRqGetDescriptor;
    req.wValue = (kUSBStringDesc << 8) | descIndex;
    req.wIndex = lang;	// English
    req.wLength = 2;
    req.pData = &desc;
	
	if(isIOUSBDeviceInterface)
		err = (*(IOUSBDeviceInterface245 **)deviceIntf)->DeviceRequest((IOUSBDeviceInterface245 **)deviceIntf, &req);
	else
		err = (*(IOUSBInterfaceInterface **)deviceIntf)->ControlRequest((IOUSBInterfaceInterface **)deviceIntf, 0, &req);
	switch(err)
	{
		case kIOReturnSuccess:
		case kIOReturnOverrun:
			//Acceptable
			break;
		case kIOUSBPipeStalled:
			//Descriptor doesn't exist
			return -2;
		default:
			//actual error
			LOG(PHIDGET_LOG_WARNING, "GetStringDescriptor failed getting length with error: 0x%08x",err);
			return -1;
	}
    
    // If the string is 0 (it happens), then just return 0 as the length
    //
    stringLen = desc[0];
    if(stringLen == 0)
    {
        return 0;
    }
    
    // OK, now that we have the string length, make a request for the full length
    //
    req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBStandard, kUSBDevice);
    req.bRequest = kUSBRqGetDescriptor;
    req.wValue = (kUSBStringDesc << 8) | descIndex;
    req.wIndex = lang;	// English
    req.wLength = stringLen;
    req.pData = buf;
	
	if(isIOUSBDeviceInterface)
		err = (*(IOUSBDeviceInterface245 **)deviceIntf)->DeviceRequest((IOUSBDeviceInterface245 **)deviceIntf, &req);
    else
		err = (*(IOUSBInterfaceInterface **)deviceIntf)->ControlRequest((IOUSBInterfaceInterface **)deviceIntf, 0, &req);
	
    if ( (err != kIOReturnSuccess) && (err != kIOReturnOverrun) )
	{
		LOG(PHIDGET_LOG_WARNING, "GetStringDescriptor failed getting string with error: 0x%08x",err);
        return -1;
	}
	return req.wLenDone;
}

int GetStringDescriptor(IOUSBDeviceInterface245 **deviceIntf, UInt8 descIndex, void *buf, UInt16 len, UInt16 lang) {
	return GetStringDescriptorMaster(deviceIntf, descIndex, buf, len, lang, TRUE);
}
int GetStringDescriptor2(IOUSBInterfaceInterface **deviceIntf, UInt8 descIndex, void *buf, UInt16 len, UInt16 lang) {
	return GetStringDescriptorMaster(deviceIntf, descIndex, buf, len, lang, FALSE);
}

//formats getStringDescriptor messages to get string descriptors based on their index value
int getStringProperty(UInt8 strIndex, IOUSBDeviceInterface245 **deviceIntf, char *str2, int str2size) {
    Byte buf[256];
	int trycount;
    
    if (strIndex > 0)
    {
        int len;
        buf[0] = 0;
		
		trycount = 0;
        while((len = GetStringDescriptor(deviceIntf, strIndex, buf, sizeof(buf),0)) == -1 || len == 0 || len != buf[0])
		{
			trycount++;
			if(trycount >= 10)
			{
				LOG(PHIDGET_LOG_ERROR, "getStringProperty failed after multiple attempts");
				return EPHIDGET_UNEXPECTED;
			}
			usleep(25000);
		}
        
        if (len > 2)
        {
            Byte *p;
            CFStringRef str;
			/* This converts to machine endianness from little endian - so does nothing on Intel */
            for (p = buf + 2; p < buf + len; p += 2)
            {
                Swap16(p);
            }
            str = CFStringCreateWithCharacters(NULL, (const UniChar *)(buf+2), (len-2)/2);
            CFStringGetCString(str, str2, str2size, kCFStringEncodingUTF8);
            CFRelease(str);
        }
        else
        {
			str2[0] = '\0';
        }
	
		//LOG(PHIDGET_LOG_INFO,"Label len: %d, string: \"%s\"",len,str2);
    }
    else
    {
		str2[0] = '\0';
    }
	
	return EPHIDGET_OK;
}

int getLabelString(void *deviceIntf, char *str2, int serialNumber, int isIOUSBDeviceInterface) {
    Byte buf[256];
	int trycount;
	int ret = EPHIDGET_OK;

	int len;
	buf[0] = 0;
	
	trycount = 0;
	while((len = GetStringDescriptorMaster(deviceIntf, 4, buf, sizeof(buf),0,isIOUSBDeviceInterface)) == -1 || len == 0 || len != buf[0])
	{
		if(len==-2)
		{
			str2[0] = '\0';
			LOG(PHIDGET_LOG_INFO, "No label string - probably an older Phidget.");
			return EPHIDGET_OK;
		}
		
		trycount++;
		if(trycount >= 10)
		{
			LOG(PHIDGET_LOG_ERROR, "getLabelString failed after multiple attempts");
			return EPHIDGET_UNEXPECTED;
		}
		usleep(25000);
	}
	
	if (len > 2)
	{
		ret = decodeLabelString((char *)buf, str2, serialNumber);
	}
	else
	{
		str2[0] = '\0';
	}
	
	//LOG(PHIDGET_LOG_INFO,"Label len: %d, string: \"%s\"",len,str2);

	return ret;
}

/*
FindInterace takes an opened DeviceInterface (from getDevice) and tries to open the first interface.
It adds the interface handle pointers to phid and returns EPHIDGET_OK if all is successfull.
*/
IOReturn FindInterface(CPhidgetHandle phid, IOUSBDeviceInterface245 **dev)
{
    IOReturn					kr;
    IOUSBFindInterfaceRequest	request;
    io_iterator_t				iterator;
    io_service_t				usbInterface;
    IOCFPlugInInterface			**plugInInterface = NULL;
	IOUSBInterfaceInterface 	**intf = NULL;
    HRESULT						res;
    SInt32						score;
	UInt8						interfaceNumber;
			
	Byte		buf1[256];
	int			len,i,trycount;
	
    request.bInterfaceClass = kIOUSBFindInterfaceDontCare;
    request.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
    request.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
    request.bAlternateSetting = kIOUSBFindInterfaceDontCare;
   
    kr = (*dev)->CreateInterfaceIterator(dev, &request, &iterator);
    while ( (usbInterface = IOIteratorNext(iterator)) != 0 ) {
   
		kr = IOCreatePlugInInterfaceForService(usbInterface, kIOUSBInterfaceUserClientTypeID, kIOCFPlugInInterfaceID, &plugInInterface, &score);
		kr = IOObjectRelease(usbInterface);				// done with the usbInterface object now that I have the plugin
		if ((kIOReturnSuccess != kr) || !plugInInterface) 
			return EPHIDGET_UNEXPECTED;
			
		// I have the interface plugin. I need the interface interface
		res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID), (LPVOID*)&intf);
		
		//This seems to leak for some reason.......
		(*plugInInterface)->Stop(plugInInterface);
		IODestroyPlugInInterface(plugInInterface);			// done with this
		
		//(*plugInInterface)->Release(plugInInterface);
		
		if (res || !intf) 
			return EPHIDGET_UNEXPECTED;
		
		kr = (*intf)->GetInterfaceNumber(intf, &interfaceNumber);
		if (kIOReturnSuccess != kr)
		{
			return EPHIDGET_UNEXPECTED;
		}
		
		if(phid->deviceDef->pdd_iid != interfaceNumber)
			continue;
		
		//find output report size from HID descriptor
		trycount = 0;
		while((len = GetDescriptorFromInterface(dev, kUSBReportDesc, 0, phid->deviceDef->pdd_iid, &buf1, sizeof(buf1))) == -1)
		{
			trycount++;
			if(trycount > 5)
			{
				(void) (*intf)->Release(intf);
				LOG(PHIDGET_LOG_ERROR, "Failed to get HID descriptor after several tries");
				return EPHIDGET_UNEXPECTED;
			}
		}
			
		phid->deviceHandle = (HANDLE)intf;
		
		if(len >= 10)
		{	
			for(i=10;i<len;i++) {
				if(buf1[i]==0x81 && buf1[i-2]==0x95)
					phid->inputReportByteLength=buf1[i-1];
				else if(buf1[i]==0x81 && buf1[i-4]==0x95)
					phid->inputReportByteLength=buf1[i-3];
				if(buf1[i]==0x91 && buf1[i-2]==0x95)
					phid->outputReportByteLength=buf1[i-1];
				else if(buf1[i]==0x91 && buf1[i-4]==0x95)
					phid->outputReportByteLength=buf1[i-3];
			}
		}
		else
		{
			LOG(PHIDGET_LOG_ERROR, "Couldn't get report lengths in CUSBGetDeviceCapabilities");
			return EPHIDGET_UNEXPECTED;
		}
		
		return EPHIDGET_OK;
	}
	return EPHIDGET_NOTFOUND;
}

int reenumerateDevice(CPhidgetHandle phid)
{		
    kern_return_t		kr;
	IOUSBInterfaceInterface **intf = (IOUSBInterfaceInterface**)phid->deviceHandle;
	//io_service_t usbDevice = (io_service_t)phid->CPhidgetFHandle;
	io_service_t usbDevice2;
	IOUSBDeviceInterface245 **dev;
    IOCFPlugInInterface 	**plugInInterface=NULL;
    HRESULT 			res;
    SInt32 			score;
	
	LOG(PHIDGET_LOG_DEBUG, "Trying to reset device...");
	
	kr = (*intf)->GetDevice(intf, &usbDevice2);
	if (kIOReturnSuccess != kr)
	{
		LOG(PHIDGET_LOG_ERROR, "Failed to get device (%08x)",kr);
		return EPHIDGET_UNEXPECTED;
	}
	
	kr = IOCreatePlugInInterfaceForService(usbDevice2, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID, &plugInInterface, &score);
	if ((kIOReturnSuccess != kr) || !plugInInterface)
	{
		printf("unable to create a plugin (%08x)\n", kr);
		return EPHIDGET_UNEXPECTED;
	}
	
	kr = IOObjectRelease(usbDevice2);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		LOG(PHIDGET_LOG_ERROR, "Failed release usbDevice (%08x)",kr);
		return EPHIDGET_UNEXPECTED;
	}
	
	// I have the device plugin, I need the device interface
	res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID), (LPVOID*)&dev);
	IODestroyPlugInInterface(plugInInterface);	
	if (res || !dev)
	{
		printf("couldn't create a device interface (%08x)\n", (int) res);
		return EPHIDGET_UNEXPECTED;
	}
	
	//need to open it first
	if (kIOReturnSuccess != (*dev)->USBDeviceOpen(dev))
	{
		(void) (*dev)->Release(dev);
		LOG(PHIDGET_LOG_ERROR, "Failed to open device (%08x)",kr);
		return EPHIDGET_UNEXPECTED;
	}
	
	kr = (*dev)->USBDeviceReEnumerate(dev, 0);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		LOG(PHIDGET_LOG_ERROR, "Failed to reset device (%08x)",kr);
		return EPHIDGET_UNEXPECTED;
	}
	
	//don't call close
	/*if (kIOReturnSuccess != (*dev)->USBDeviceClose(dev))
	{
		(void) (*dev)->Release(dev);
		LOG(PHIDGET_LOG_ERROR, "Failed to close device (%08x)",kr);
		return EPHIDGET_UNEXPECTED;
	}*/
	
	return EPHIDGET_OK;
}

/*
getDevice takes an IORegistry entry pointer and verifies that it is
accessible and that the serial number matches if it needs to and it
instantiates phid
*/
int getDevice(CPhidgetHandle phid, io_service_t usbDevice) {
    kern_return_t		kr;
    IOCFPlugInInterface 	**plugInInterface=NULL;
    HRESULT 			res;
    SInt32 			score;
    UInt16			vendor;
    UInt16			product;
    UInt16			release;
	UInt8 temp;
	int i;
	char serialnumstr[50];
	char label[MAX_LABEL_STORAGE];
	IOUSBConfigurationDescriptorPtr configDesc;
	UInt8 configIndex = 0;
	
	IOUSBDeviceInterface245 **dev = NULL;

	kr = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID, &plugInInterface, &score);
	//don't release this because we need it for when the device gets removed
	//kr = IOObjectRelease(usbDevice);
	
	if ((kIOReturnSuccess != kr) || !plugInInterface)
	{
		printf("unable to create a plugin (%08x)\n", kr);
		return EPHIDGET_UNEXPECTED;
	}
		
	// I have the device plugin, I need the device interface
	res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID), (LPVOID*)&dev);
	
	(*plugInInterface)->Stop(plugInInterface);
	IODestroyPlugInInterface(plugInInterface);	
	
	if (res || !dev)
	{
		printf("couldn't create a device interface (%08x)\n", (int) res);
		return EPHIDGET_UNEXPECTED;
	}
	
	kr = (*dev)->GetConfiguration(dev, &configIndex);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}
	
	kr = (*dev)->GetConfigurationDescriptorPtr(dev, 0, &configDesc);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}
	
	IOUSBInterfaceDescriptorPtr interfaceDesc = NULL;
	IOUSBConfigurationDescHeaderPtr currentDesc;
	unsigned char *rawDescriptorData = (void *)configDesc;
	
	//Find the interface Descriptor
	for(i=0;i<configDesc->wTotalLength-2;i+=currentDesc->bLength)
	{
		currentDesc = (IOUSBConfigurationDescHeaderPtr)&rawDescriptorData[i];
		if (currentDesc->bDescriptorType == 0x04)
		{
			interfaceDesc = (IOUSBInterfaceDescriptorPtr)&rawDescriptorData[i];
			if(interfaceDesc->bInterfaceNumber == phid->deviceDef->pdd_iid)
				break;
			else
				interfaceDesc = NULL;
		}
	}
	
	if(interfaceDesc == NULL)
	{
		LOG(PHIDGET_LOG_ERROR, "Couldn't find interface descriptor!");
		return EPHIDGET_UNEXPECTED;
	}
	
	if(interfaceDesc->bNumEndpoints == 2)
	{
		LOG(PHIDGET_LOG_INFO, "Using Interrupt OUT Endpoint for Host->Device communication.");
		phid->interruptOutEndpoint = PTRUE;
	}
	else
	{
		LOG(PHIDGET_LOG_INFO, "Using Control Endpoint for Host->Device communication.");
		phid->interruptOutEndpoint = PFALSE;
	}
	
	kr = (*dev)->GetDeviceVendor(dev, &vendor);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}
	kr = (*dev)->GetDeviceProduct(dev, &product);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}
	kr = (*dev)->GetDeviceReleaseNumber(dev, &release);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}

	//fill in the phid info
	phid->ProductID = product;
	phid->VendorID = vendor;
		
	if (phid->deviceDef->pdd_pid == phid->ProductID && phid->deviceDef->pdd_vid == phid->VendorID) {
		phid->deviceType = (char *)Phid_DeviceName[phid->deviceID];
	}
	else
		return EPHIDGET_NOTFOUND;	

	kr = (*dev)->USBGetSerialNumberStringIndex(dev, &temp);
	if (kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}
	if(getStringProperty(temp, dev, serialnumstr, 50))
	{
		LOG(PHIDGET_LOG_ERROR, "Failed to get serial number string");
		return EPHIDGET_UNEXPECTED;
	}
	
	if (phid->specificDevice == PHIDGETOPEN_SERIAL)
	{
		if (phid->serialNumber != atoi(serialnumstr))
			return EPHIDGET_NOTFOUND;
	}

	phid->serialNumber = atoi(serialnumstr);
	
	
	//get the product string
	kr = (*dev)->USBGetProductStringIndex(dev, &temp);
	if(kIOReturnSuccess != kr)
	{
		(void) (*dev)->Release(dev);
		return EPHIDGET_UNEXPECTED;
	}
	if(getStringProperty(temp, dev, phid->usbProduct, 64))
	{
		LOG(PHIDGET_LOG_ERROR, "Failed to get Product String");
		return EPHIDGET_UNEXPECTED;
	}
	
	
	//get the label string
	if(getLabelString(dev, label, phid->serialNumber, TRUE))
	{
		LOG(PHIDGET_LOG_INFO, "Failed to get label string - probably an older Phidget.");
		label[0] = '\0';
	}
	
	if (phid->specificDevice == PHIDGETOPEN_LABEL)
	{
		if (strncmp(phid->label, label, MAX_LABEL_STORAGE-1))
			return EPHIDGET_NOTFOUND;
	}
	
	memcpy(phid->label, label, MAX_LABEL_STORAGE);

	if (release < 0x100)
		phid->deviceVersion = release * 100;
	else
		phid->deviceVersion = ((release >> 8) * 100) + ((release & 0xff));
			
	phid->deviceUID = CPhidget_getUID(phid->deviceIDSpec, phid->deviceVersion);
	
	phid->attr = phid->deviceDef->pdd_attr;
	
	//device has been verified, now get the interface
	kr = FindInterface(phid, dev);
	if(kr) 
	{
		return kr;
	}
	
	kr = (*dev)->Release(dev);
	return EPHIDGET_OK;
}

int refreshLabelString(CPhidgetHandle phid)
{
	IOUSBInterfaceInterface **intf = (IOUSBInterfaceInterface**)phid->deviceHandle;
	char label[MAX_LABEL_STORAGE] = {0};
	
	//get the label string
	if(getLabelString(intf, label, phid->serialNumber, FALSE))
	{
		LOG(PHIDGET_LOG_ERROR, "Failed to get label string");
		return EPHIDGET_UNEXPECTED;
	}
	
	memcpy(phid->label, label, MAX_LABEL_STORAGE);
	
	return EPHIDGET_OK;
}

/*
This is a notification function called when a Phidget has matched the kernel driver
It will instantiate and verify the Phidget and add it to the Phidget list for Phidget Manager
*/
void RawDeviceAdded(void *refCon, io_iterator_t iterator)
{
    io_service_t	usbDevice;
	CPhidgetDeviceDef *deviceDef;
	CPhidgetHandle phid;
	
	deviceDef = refCon;
	
	//add it to the list
	while ( (usbDevice = IOIteratorNext(iterator)) )
	{
		io_name_t className;
		IOObjectGetClass(usbDevice, className);
	
		LOG(PHIDGET_LOG_INFO, "Attach: %ld\n", (long)usbDevice);
		CPhidget_create(&phid);
		phid->deviceID = deviceDef->pdd_did;
		phid->deviceIDSpec = deviceDef->pdd_sdid;
		phid->deviceDef = deviceDef;
		phid->serialNumber = -1;
		if(getDevice(phid, usbDevice)) {
			CPhidget_free(phid);
			LOG(PHIDGET_LOG_WARNING, "getDevice failed in RawDeviceAdded");
			continue;
		}
		phid->CPhidgetFHandle = usbDevice;
		CPhidget_setStatusFlag(&phid->status, PHIDGET_ATTACHED_FLAG, &phid->lock);

		//run the attach handler
		phid->specificDevice = TRUE;
		CThread_mutex_lock(&attachedDevicesLock);
		CPhidgetAttachEvent(phid);
		CThread_mutex_unlock(&attachedDevicesLock);
	}
}

/*
This is a notification function that gets called when a Phidget is unplugged
It will remove that Phidget from the Phidget list
*/
void RawDeviceRemoved(void *refCon, io_iterator_t iterator)
{
    io_service_t	obj;
	CPhidgetList *trav = 0, *detachList = 0;

	CThread_mutex_lock(&attachedDevicesLock);
    while ( (obj = IOIteratorNext(iterator)) )
    {
		LOG(PHIDGET_LOG_INFO, "Detach: %ld\n", (long)obj);
		for (trav=AttachedDevices; trav; trav = trav->next)
		{
			if(IOObjectIsEqualTo(obj, trav->phid->CPhidgetFHandle))
			{
				CList_addToList((CListHandle *)&detachList, trav->phid, CPhidget_areEqual);
				break;
			}
		}
		IOObjectRelease(obj);
	}
	//run the phidget manager detach handlers
	for (trav=detachList; trav; trav = trav->next)
	{
		trav->phid->specificDevice = TRUE;
		CPhidgetDetachEvent(trav->phid);
	}
	CList_emptyList((CListHandle *)&detachList, PFALSE, NULL);
	CThread_mutex_unlock(&attachedDevicesLock);
}

void SignalHandler(int sigraised)
{
    //printf("\nInterrupted\n");
	CPhidgetManager_teardownNotifications();

    // exit(0) should not be called from a signal handler.  Use _exit(0) instead
    //
    _exit(0);
}

#ifdef POWER_CALLBACKS

void
MySleepCallBack( void * refCon, io_service_t service, natural_t messageType, void * messageArgument )
{
    /*printf( "messageType %08lx, arg %08lx\n",
		   (long unsigned int)messageType,
		   (long unsigned int)messageArgument );*/
	
    switch ( messageType )
    {

        case kIOMessageCanSystemSleep:
        case kIOMessageSystemWillSleep:
			{
				CPhidgetList *trav;
				int waitTime = 3000;
				int attachedDevices = PFALSE;
				
				//Lets the user access Phidgets before sleep
				if (fptrSleep)
					fptrSleep(fptrSleepptr);
				
				//Pause USB traffic
				LOG(PHIDGET_LOG_INFO, "Pausing USB traffic in preparation for system sleep...");
				
				pause_usb_traffic = PTRUE;
				
				for (trav=ActiveDevices; trav; trav = trav->next)
				{
					if(CPhidget_statusFlagIsSet(trav->phid->status, PHIDGET_ATTACHED_FLAG))
					{
						attachedDevices = PTRUE;
					}
				}
				
				// wait for reads/writes to stop 
				if(attachedDevices)
					while((!usb_read_paused || !usb_write_paused) && waitTime > 0)
					{
						SLEEP(10);
						waitTime-=10;
					}
				
				LOG(PHIDGET_LOG_INFO, "USB traffic paused, now sleeping...");
				
				//resume sleeping
				IOAllowPowerChange( root_port, (long)messageArgument );
			}
            break;
			
        case kIOMessageSystemWillPowerOn:
            //System has started the wake up process...
            break;
			
        case kIOMessageSystemHasPoweredOn:
            //Resume USB traffic
			LOG(PHIDGET_LOG_INFO, "Resuming USB traffic on system wake...");
			
			pause_usb_traffic = PFALSE;
			
			//Lets the user access Phidgets after wakeup
			if (fptrWakeup)
				fptrWakeup(fptrWakeupptr);
			
			break;
			
        default:
            break;
    }
}
#endif

int CCONV
CPhidgetManager_teardownNotifications()
{
	if(!runLoopRef)
		return 0;
#ifdef POWER_CALLBACKS
	// remove the sleep notification port from the application runloop
    CFRunLoopRemoveSource( runLoopRef,
						  IONotificationPortGetRunLoopSource(notifyPortRef),
						  kCFRunLoopCommonModes );
	
    // deregister for system sleep notifications
    IODeregisterForSystemPower( &notifierObject );
	
    // IORegisterForSystemPower implicitly opens the Root Power Domain IOService
    // so we close it here
    IOServiceClose( root_port );
	
    IONotificationPortDestroy(notifyPortRef);
#endif
	
	//deregister phidgets
    CFRunLoopRemoveSource( runLoopRef,
						  IONotificationPortGetRunLoopSource(gNotifyPort),
						  kCFRunLoopDefaultMode );
	
    // Clean up here
    IONotificationPortDestroy(gNotifyPort);
	
    if (gRawAddedIter) 
    {
        IOObjectRelease(gRawAddedIter);
        gRawAddedIter = 0;
    }
	
    if (gRawRemovedIter) 
    {
        IOObjectRelease(gRawRemovedIter);
        gRawRemovedIter = 0;
    }
	
	runLoopRef = NULL;
	
	return 0;
}

/*
Sets up notifications for adding and removing Phidgets.

Right now: on Phidget add, RawDeviceAdded is called, on remove, RawDeviceRemoved is called
Calling this function will block! as the notifications happen within the run loop.
*/
int CCONV
CPhidgetManager_setupNotifications(CFRunLoopRef runloop) {
	mach_port_t				masterPort;
    CFMutableDictionaryRef 	matchingDict[PHIDGET_DEVICE_COUNT];
	SInt32					usbVendor;
    SInt32					usbProduct;
    kern_return_t			kr;
	int						i;
	sig_t					oldHandler;
	CFNumberRef				vendor, product;
	
	runLoopRef = runloop;
	
	// Set up a signal handler so we can clean up when we're interrupted
    // Otherwise we stay in our run loop forever.
    oldHandler = signal(SIGINT, SignalHandler);
    if (oldHandler == SIG_ERR)
        printf("Could not establish new signal handler");
	
    // first create a master_port for my task
    kr = IOMasterPort(MACH_PORT_NULL, &masterPort);
    if (kr || !masterPort)
    {
        printf("ERR: Couldn't create a master IOKit Port(%08x)\n", kr);
        return -1;
    }
	
#ifdef POWER_CALLBACKS
	//register for power notifications
	root_port = IORegisterForSystemPower( NULL, &notifyPortRef, MySleepCallBack, &notifierObject );
    if ( root_port == 0 )
    {
        printf("IORegisterForSystemPower failed\n");
        return -1;
	}
    
	// add the notification port to the application runloop
    CFRunLoopAddSource(runloop, IONotificationPortGetRunLoopSource(notifyPortRef), kCFRunLoopDefaultMode ); 
#endif
	
	// Create a notification port and add its run loop event source to our run loop
	// This is how async notifications get set up.
	gNotifyPort = IONotificationPortCreate(masterPort);
	CFRunLoopAddSource(runloop, IONotificationPortGetRunLoopSource(gNotifyPort), kCFRunLoopDefaultMode);

	for (i = 1; i<PHIDGET_DEVICE_COUNT; i++) {
		usbProduct = Phid_Device_Def[i].pdd_pid;
		usbVendor = Phid_Device_Def[i].pdd_vid;
		//deviceSpec[i] = Phid_Device_Def[i].pdd_sdid;

		// Set up the matching criteria for the devices we're interested in
		matchingDict[i] = IOServiceMatching(kIOUSBDeviceClassName);	// Interested in instances of class IOUSBDevice and its subclasses
		if (!matchingDict[i])
		{
			printf("Can't create a USB matching dictionary\n");
			mach_port_deallocate(mach_task_self(), masterPort);
			return -1;
		}
		
		// Add our vendor and product IDs to the matching criteria
		vendor = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbVendor);
		product = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &usbProduct);
		CFDictionarySetValue( 
				matchingDict[i], 
				CFSTR(kUSBVendorID), 
				vendor); 
		CFDictionarySetValue( 
				matchingDict[i], 
				CFSTR(kUSBProductID), 
				product);
		CFRelease(vendor);
		CFRelease(product);
		
		// Retain additional references because we use this same dictionary with 2 calls to 
		// IOServiceAddMatchingNotification, each of which consumes one reference.
		matchingDict[i] = (CFMutableDictionaryRef) CFRetain( matchingDict[i] );
		
		// Now set up two notifications, one to be called when a raw device is first matched by I/O Kit, and the other to be
		// called when the device is terminated.
		kr = IOServiceAddMatchingNotification(  gNotifyPort,
												kIOFirstMatchNotification,
												matchingDict[i],
												RawDeviceAdded,
												(void *)&Phid_Device_Def[i],
												&gRawAddedIter );
		RawDeviceAdded((void *)&Phid_Device_Def[i], gRawAddedIter);	// Iterate once to get already-present devices and
                                                // arm the notification   

		kr = IOServiceAddMatchingNotification(  gNotifyPort,
												kIOTerminatedNotification,
												matchingDict[i],
												RawDeviceRemoved,
												(void *)&Phid_Device_Def[i],
												&gRawRemovedIter );
												
		RawDeviceRemoved((void *)&Phid_Device_Def[i], gRawRemovedIter);	// Iterate once to arm the notification		
	}

    // Now done with the master_port
    mach_port_deallocate(mach_task_self(), masterPort);
    masterPort = 0;

    return 0;
}
