/*
 * This file is part of libphidget21
 *
 * Copyright 2006-2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see 
 * <http://www.gnu.org/licenses/>
 */

#include "stdafx.h"
#ifdef __CYGWIN__
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif
#ifdef _MSC_EXTENSIONS
#include <winsock2.h>
#endif
#include <stdlib.h>
#include <string.h>
#include "getaddrinfo.h"

typedef enum {
	EAI_SERVICE = 1,
	EAI_FAMILY,
	EAI_SOCKTYPE,
	EAI_NODATA,
	EAI_MEMORY
} gai_err_t;
#ifdef __CYGWIN__
int
getaddrinfo(const char *nodename, const char  *servname,
    const struct addrinfo *hints, struct addrinfo **res)
{
	struct hostent *h;
	struct addrinfo *ret;
	struct sockaddr_in *sin;

	if (!(atoi(servname)))
		return EAI_SERVICE;
	if (hints->ai_family != AF_INET && hints->ai_family != PF_UNSPEC)
		return EAI_FAMILY;
	if (hints->ai_socktype != SOCK_STREAM)
		return EAI_SOCKTYPE;
	if (!(h = gethostbyname(nodename)) || !h->h_addr_list[0])
		return EAI_NODATA;
	if (!(ret = malloc(sizeof (*ret))))
		return EAI_MEMORY;
	memset(ret, 0, sizeof (*ret));
	ret->ai_family= AF_INET;
	ret->ai_socktype = SOCK_STREAM;
	ret->ai_protocol = IPPROTO_TCP;
	ret->ai_addrlen = sizeof (struct sockaddr_in);
	ret->ai_addr = (struct sockaddr *)sin;
	sin->sin_family = AF_INET;
	sin->sin_port = htons(atoi(servname));
	memcpy(&sin->sin_addr, h->h_addr_list[0], sizeof (sin->sin_addr));
	return 0;
}

void
freeaddrinfo(struct addrinfo *ai)
{
	struct addrinfo *next;

	do {
		free(ai->ai_addr);
		next = ai->ai_next;
		free(ai);
		ai = next;
	} while (ai);
}
#endif

#ifndef _MSC_EXTENSIONS
char *
gai_strerror(int code)
{
	switch (code) {
	case EAI_FAMILY:
		return "Unsupported family";
	case EAI_MEMORY:
		return "Out of memory";
	case EAI_NODATA:
		return "No address is associated with nodename";
	case EAI_SERVICE:
		return "Unsupported service (use numeric specification)";
	case EAI_SOCKTYPE:
		return "Unsupported socket type";
	}
	return "Unknown error";
}
#endif
