/*
 * This file is part of libphidget21
 *
 * Copyright 2006-2015 Phidgets Inc <patrick@phidgets.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see 
 * <http://www.gnu.org/licenses/>
 */

#ifndef __WINCOMPAT_H
#define __WINCOMPAT_H

#ifdef _MSC_EXTENSIONS
#include "snprintf.h"
#include "regex/regex.h"
#endif

#ifndef EXTERNALPROTO

/* if this is used on Windows, you need to call pthread_mutex_init()!! */
#define PTHREAD_MUTEX_INITIALIZER NULL

typedef struct pthread *pthread_t;
typedef struct pthread_attr pthread_attr_t;
typedef struct pthread_cond *pthread_cond_t;
typedef struct pthread_condattr pthread_condattr_t;
typedef struct pthread_mutex *pthread_mutex_t;
typedef struct pthread_mutexattr pthread_mutexattr_t;

extern int pthread_create(pthread_t *, const pthread_attr_t *,
    void * (*)(void *), void *);
extern int pthread_join(pthread_t, void **);
extern int pthread_mutex_init(pthread_mutex_t *, const pthread_mutexattr_t *);
extern int pthread_mutex_destroy(pthread_mutex_t *);
extern int pthread_mutex_lock(pthread_mutex_t *);
extern int pthread_mutex_unlock(pthread_mutex_t *);
extern int pthread_cond_destroy(pthread_cond_t *);
extern int pthread_cond_init(pthread_cond_t *, const pthread_condattr_t *);
extern int pthread_cond_wait(pthread_cond_t *, pthread_mutex_t *);
extern int pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, int *timeout);
extern int pthread_cond_signal(pthread_cond_t *);
extern pthread_t pthread_self(void);

int socketpair(int domain, int type, int protocol, int sv[2]);

int
strcasecmp(const char *s1, const char *s2);

int usleep(unsigned long useconds);

#if ((defined(_WINDOWS) && defined(__GNUC__)) || (defined(WINCE) && _WIN32_WCE < 0x600))
char *strdup(const char *);
#endif
#if defined(WINCE)
__int64 _strtoi64(
   const char *nptr,
   char **endptr,
   int base 
);

#endif

#ifndef WINCE
#include <errno.h>
#endif

#ifndef ETIMEDOUT
#define ETIMEDOUT WAIT_TIMEOUT
#endif

#ifndef EINVAL
#define EINVAL WAIT_FAILED
#endif

#endif
#endif
